@@ -9,7 +9,7 @@
 ** /dev/random for salt generation added by 
 ** Aaron Sethman <androsyn@ratbox.org>
 **
-** $Id: mkpasswd.c,v 1.6 2003/10/18 17:26:36 q Exp $
+** $Id: mkpasswd.c,v 1.7 2005/01/03 22:16:59 q Exp $
 */
 #include <stdio.h>
 #include <string.h>
@@ -183,7 +183,7 @@ int main(int argc, char *argv[])
   return 0;
 }
 
-char *make_des_salt()
+char *make_des_salt(void)
 {
   static char salt[3];
   generate_random_salt(salt, 2);
@@ -332,7 +332,7 @@ char *generate_random_salt(char *salt, int length)
   return(salt);
 }
 
-void full_usage()
+void full_usage(void)
 {
   printf("mkpasswd [-m|-d|-b|-e] [-l saltlength] [-r rounds] [-s salt] [-p plaintext]\n");
   printf("-m Generate an MD5 password\n");
@@ -350,7 +350,7 @@ void full_usage()
   exit(0);
 }
 
-void brief_usage()
+void brief_usage(void)
 {
   printf("mkpasswd - password hash generator\n");
   printf("Standard DES:  mkpasswd [-d] [-s salt] [-p plaintext]\n");
