@@ -18,7 +18,7 @@
  */
 
 #ifndef lint
-static const volatile char rcsid[] = "@(#)$Id: chkconf.c,v 1.42 2004/12/06 17:07:23 chopin Exp $";
+static const volatile char rcsid[] = "@(#)$Id: chkconf.c,v 1.51 2008/06/24 22:24:52 chopin Exp $";
 #endif
 
 #include "os.h"
@@ -27,6 +27,10 @@ static const volatile char rcsid[] = "@(#)$Id: chkconf.c,v 1.42 2004/12/06 17:07
 #include "match_ext.h"
 #undef CHKCONF_C
 
+#define mystrdup(x) strdup(x)
+#ifdef MyMalloc
+#undef MyMalloc
+#endif
 #define MyMalloc(x)     malloc(x)
 /*#define MyFree(x)       free(x)*/
 
@@ -142,10 +146,18 @@ int	main(int argc, char *argv[])
  * configuration file from.  This may either be th4 file direct or one end
  * of a pipe from m4.
  */
-static	int	openconf()
+static	int	openconf(void)
 {
 #ifdef	M4_PREPROC
 	int	pi[2];
+#ifdef HAVE_GNU_M4
+	char	*includedir, *includedirptr;
+
+	includedir = strdup(IRCDM4_PATH);
+	includedirptr = strrchr(includedir, '/');
+	if (includedirptr)
+		*includedirptr = '\0';
+#endif
 
 	/* ircd.m4 with full path now! Kratz */
 	if (access(IRCDM4_PATH, R_OK) == -1)
@@ -174,7 +186,17 @@ static	int	openconf()
 		 * goes out with report_error.  Could be dangerous,
 		 * two servers running with the same fd's >:-) -avalon
 		 */
-		(void)execlp(M4_PATH, "m4", IRCDM4_PATH, configfile, 0);
+		(void)execlp(M4_PATH, "m4",
+#ifdef HAVE_GNU_M4
+#ifdef USE_M4_PREFIXES
+			"-P",
+#endif
+			"-I", includedir,
+#endif
+#ifdef INET6
+			"-DINET6",
+#endif
+			IRCDM4_PATH, configfile, 0);
 		perror("m4");
 		exit(-1);
 	default :
@@ -187,7 +209,7 @@ static	int	openconf()
 }
 
 /* show config as ircd would see it before starting to parse it */
-static	void	showconf()
+static	void	showconf(void)
 {
 #if defined(CONFIG_DIRECTIVE_INCLUDE)
 	aConfig *p, *p2;
@@ -255,7 +277,7 @@ static	void	showconf()
 **            NULL if config is invalid (some mandatory fields missing)
 */
 
-static	aConfItem 	*initconf()
+static	aConfItem 	*initconf(void)
 {
 	int	fd;
 	char	*tmp, *tmp3 = NULL, *s;
@@ -526,6 +548,10 @@ static	aConfItem 	*initconf()
 				case 'M':
 				case 'F':
 					break;
+				case ' ':
+				case '\t':
+					/* so there's no weird warnings */
+					break;
 				default:
 					config_error(CF_WARN, CK_FILE, CK_LINE,
 						"unknown I-line flag: %c", *s);
@@ -562,6 +588,10 @@ static	aConfItem 	*initconf()
 				case 'P':
 				case 't':
 					break;
+				case ' ':
+				case '\t':
+					/* so there's no weird warnings */
+					break;
 				default:
 					config_error(CF_WARN, CK_FILE, CK_LINE,
 						"unknown O-line flag: %c", *s);
@@ -1012,9 +1042,11 @@ static	void	validate(aConfItem *top)
 			nr = aconf->clients;
 			filelist = findConfLineNumber(nr);
 			config_error(CF_WARN, CK_FILE, CK_LINE,
-				"unmatched %c:%s:%s:%s",
-				confchar(aconf->status), aconf->host,
-				SHOWSTR(aconf->passwd), aconf->name);
+				"unmatched %c%c%s%c%s%c%s",
+				confchar(aconf->status), IRCDCONF_DELIMITER,
+				aconf->host, IRCDCONF_DELIMITER,
+				SHOWSTR(aconf->passwd), IRCDCONF_DELIMITER,
+				aconf->name);
 		    }
 	return;
 }
@@ -1092,7 +1124,7 @@ static int simulateM4Include(struct wordcount *filelist, int nr, char *filename,
 #endif
 
 #ifndef CONFIG_DIRECTIVE_INCLUDE
-static void	mywc()
+static void	mywc(void)
 {
 	int	fd, dh, nr = 0;
 	char	line[512];
