@@ -19,7 +19,7 @@
  */
 
 #ifndef lint
-static const volatile char rcsid[] = "@(#)$Id: ircd.c,v 1.153 2004/11/16 16:56:16 chopin Exp $";
+static const volatile char rcsid[] = "@(#)$Id: ircd.c,v 1.165 2010/08/11 17:39:00 bif Exp $";
 #endif
 
 #include "os.h"
@@ -81,6 +81,22 @@ RETSIGTYPE s_die(int s)
 	logfiles_close();
 	ircd_writetune(tunefile);
 	flush_connections(me.fd);
+#ifdef  UNIXPORT
+	{
+		aClient *acptr;
+		char unixpath[256];
+		for (acptr = ListenerLL; acptr; acptr = acptr->next)
+		{
+			if (IsUnixSocket(acptr))
+			{
+				sprintf(unixpath, "%s/%d",
+				    acptr->confs->value.aconf->host,
+				    acptr->confs->value.aconf->port);
+				(void)unlink(unixpath);
+			}
+		}
+	}
+#endif
 	exit(-1);
 }
 
@@ -115,7 +131,10 @@ static RETSIGTYPE s_rehash(int s)
 #else
 	(void)signal(SIGHUP, s_rehash);	/* sysV -argv */
 #endif
-	dorehash = 1;
+	if (dorehash >= 1)
+		dorehash = 2;
+	else
+		dorehash = 1;
 }
 
 void	restart(char *mesg)
@@ -152,7 +171,7 @@ RETSIGTYPE s_restart(int s)
 	dorestart = 1;
 }
 
-void	server_reboot()
+void	server_reboot(void)
 {
 	Reg	int	i;
 
@@ -219,6 +238,9 @@ static	time_t	try_connections(time_t currenttime)
 	int	i;
 #endif
 
+	if ((bootopt & BOOT_STANDALONE))
+		return 0;
+
 	Debug((DEBUG_NOTICE,"Connection check at   : %s",
 		myctime(currenttime)));
 	for (aconf = conf; aconf; aconf = aconf->next )
@@ -231,6 +253,11 @@ static	time_t	try_connections(time_t currenttime)
 		if (aconf->port <= 0)
 			continue;
 
+		cltmp = Class(aconf);
+		/* not a candidate for AC */
+		if (MaxLinks(cltmp) == 0)
+			continue;
+
 		/* minimize next to lowest hold time of all AC-able C-lines */
 		if (next > aconf->hold || next == 0)
 			next = aconf->hold;
@@ -242,7 +269,6 @@ static	time_t	try_connections(time_t currenttime)
 		/* at least one candidate not held for future, good */
 		allheld = 0;
 
-		cltmp = Class(aconf);
 		/* see if another link in this conf is allowed */
 		if (Links(cltmp) >= MaxLinks(cltmp))
 			continue;
@@ -330,7 +356,8 @@ static	time_t	try_connections(time_t currenttime)
 		 * try_connections() other servers have chance. --B. */
 		con_conf->hold += get_con_freq(Class(con_conf));
 
-		if (!iconf.aconnect)
+		if (iconf.aconnect == 0 || iconf.aconnect == 2 && 
+				timeofday - iconf.split > DELAYCHASETIMELIMIT)
 		{
 			sendto_flag(SCH_NOTICE,
 				"Connection to %s deferred. Autoconnect "
@@ -492,7 +519,7 @@ static	time_t	check_pings(time_t currenttime)
 	aClient *bysptr = NULL;
 	int	ping = 0, i;
 	time_t	oldest = 0, timeout;
-	char	*reason;
+	char	*reason = NULL;
 
 	for (i = highest_fd; i >= 0; i--)
 	    {
@@ -848,6 +875,8 @@ int	main(int argc, char *argv[])
 				bootopt |= BOOT_PROT;
 			else if (!strcmp(p, "off"))
 				bootopt &= ~(BOOT_PROT|BOOT_STRICTPROT);
+			else if (!strcmp(p, "standalone"))
+				bootopt |= BOOT_STANDALONE;
 			else
 				bad_command();
 			break;
@@ -864,13 +893,14 @@ int	main(int argc, char *argv[])
 			tunefile = p;
 			break;
 		    case 'v':
-			(void)printf("ircd %s %s\n\tzlib %s\n\t%s #%s\n",
+			(void)printf("ircd %s %s\n\tzlib %s\n\tircd.conf delimiter %c\n\t%s #%s\n",
 				     version, serveropts,
 #ifndef	ZIP_LINKS
 				     "not used",
 #else
 				     zlib_version,
 #endif
+					IRCDCONF_DELIMITER,
 				     creation, generation);
 			  exit(0);
 		    case 'x':
@@ -1024,10 +1054,10 @@ int	main(int argc, char *argv[])
 			"Fatal Error: No M-line in ircd.conf.\n");
 			exit(-1);
 		}
-		if (check_servername(ME))
+		if ((i=check_servername(ME)))
 		{
 			fprintf(stderr,
-			"Fatal Error: Invalid server name.\n");
+			"Fatal Error: %s.\n", check_servername_errors[i-1][1]);
 			exit(-1);
 		}
 		if (!me.serv->sid)
@@ -1238,11 +1268,11 @@ static	void	io_loop(void)
 
 	if (dorestart)
 		restart("Caught SIGINT");
-	if (dorehash)
+	if (dorehash > 0)
 	    {	/* Only on signal, not on oper /rehash */
 		ircd_writetune(tunefile);
 		(void)rehash(&me, &me, 1);
-		dorehash = 0;
+		dorehash--;
 	    }
 	if (restart_iauth || timeofday >= nextiarestart)
 	    {
@@ -1273,7 +1303,7 @@ static	void	io_loop(void)
  * set from the command line by -x, use /dev/null as the dummy logfile as long
  * as DEBUGMODE has been defined, else don't waste the fd.
  */
-static	void	open_debugfile()
+static	void	open_debugfile(void)
 {
 #ifdef	DEBUGMODE
 	int	fd;
@@ -1303,7 +1333,7 @@ static	void	open_debugfile()
 	return;
 }
 
-static	void	setup_signals()
+static	void	setup_signals(void)
 {
 #if POSIX_SIGNALS
 	struct	sigaction act;
@@ -1332,8 +1362,12 @@ static	void	setup_signals()
 	(void)sigaction(SIGTERM, &act, NULL);
 # if defined(USE_IAUTH)
 	act.sa_handler = s_slave;
+# else
+	act.sa_handler = SIG_IGN;
+# endif
 	(void)sigaddset(&act.sa_mask, SIGUSR1);
 	(void)sigaction(SIGUSR1, &act, NULL);
+# if defined(USE_IAUTH)
 	act.sa_handler = SIG_IGN;
 #  ifdef SA_NOCLDWAIT
         act.sa_flags = SA_NOCLDWAIT;
@@ -1374,6 +1408,8 @@ static	void	setup_signals()
 # if defined(USE_IAUTH)
 	(void)signal(SIGUSR1, s_slave);
 	(void)signal(SIGCHLD, SIG_IGN);
+# else
+	(void)signal(SIGUSR1, SIG_IGN);
 # endif
 	
 # if defined(__FreeBSD__)
@@ -1410,7 +1446,7 @@ void ircd_writetune(char *filename)
 	    {
 		(void)sprintf(buf, "%d\n%d\n%d\n%d\n%d\n%d\n%d\n", ww_size,
 			       lk_size, _HASHSIZE, _CHANNELHASHSIZE,
-			       _SERVERSIZE, poolsize, _UIDSIZE);
+			       _SIDSIZE, poolsize, _UIDSIZE);
 		if (write(fd, buf, strlen(buf)) == -1)
 			sendto_flag(SCH_ERROR,
 				    "Failed (%d) to write tune file: %s.",
@@ -1460,10 +1496,15 @@ void ircd_readtune(char *filename)
 		ww_size = t_data[0];
 		lk_size = t_data[1];
 		_HASHSIZE = t_data[2];
+#ifdef USE_HOSTHASH
 		_HOSTNAMEHASHSIZE = t_data[2]; /* hostname has always same size
 						  as the client hash */
+#endif
+#ifdef USE_IPHASH
+		_IPHASHSIZE = t_data[2];
+#endif
 		_CHANNELHASHSIZE = t_data[3];
-		_SERVERSIZE = t_data[4];
+		_SIDSIZE = t_data[4];
 		poolsize = t_data[5];
 		_UIDSIZE = t_data[6];
 
