@@ -48,7 +48,7 @@
  */
 
 #ifndef lint
-static const volatile char rcsid[] = "@(#)$Id: s_conf.c,v 1.153 2004/12/06 17:07:22 chopin Exp $";
+static const volatile char rcsid[] = "@(#)$Id: s_conf.c,v 1.195 2010/08/11 17:16:51 bif Exp $";
 #endif
 
 #include "os.h"
@@ -56,6 +56,9 @@ static const volatile char rcsid[] = "@(#)$Id: s_conf.c,v 1.153 2004/12/06 17:07
 #define S_CONF_C
 #include "s_externs.h"
 #undef S_CONF_C
+#ifdef ENABLE_CIDR_LIMITS
+#include "patricia_ext.h"
+#endif
 
 #ifdef TIMEDKLINES
 static	int	check_time_interval (char *, char *);
@@ -255,6 +258,8 @@ char	*oline_flags_to_string(long flags)
 		*s++ ='e';
 	if (flags & ACL_TKLINE)
 		*s++ ='T';
+	if (flags & ACL_KLINE)
+		*s++ ='q';
 #ifdef CLIENTS_CHANNEL
 	if (flags & ACL_CLIENTS)
 		*s++ ='&';
@@ -265,6 +270,10 @@ char	*oline_flags_to_string(long flags)
 		*s++ = 'p';
 	if (flags & ACL_TRACE)
 		*s++ = 't';
+#ifdef ENABLE_SIDTRACE
+	if (flags & ACL_SIDTRACE)
+		*s++ = 'v';
+#endif
 	if (s == ofsbuf)
 		*s++ = '-';
 	*s++ = '\0';
@@ -298,12 +307,16 @@ long	oline_flags_parse(char *string)
 		case 'D': tmp |= ACL_DIE; break;
 		case 'e': tmp |= ACL_SET; break;
 		case 'T': tmp |= ACL_TKLINE; break;
+		case 'q': tmp |= ACL_KLINE; break;
 #ifdef CLIENTS_CHANNEL
 		case '&': tmp |= ACL_CLIENTS; break;
 #endif
 		case 'P': tmp |= ACL_NOPENALTY; break;
 		case 'p': tmp |= ACL_CANFLOOD; break;
 		case 't': tmp |= ACL_TRACE; break;
+#ifdef ENABLE_SIDTRACE
+		case 'v': tmp |= ACL_SIDTRACE; break;
+#endif
 		}
 	}
 	if (tmp & ACL_LOCOP)
@@ -321,6 +334,9 @@ long	oline_flags_parse(char *string)
 #ifndef OPER_SQUIT
 	tmp &= ~ACL_SQUIT;
 #endif
+#ifndef OPER_SQUIT_REMOTE
+	tmp &= ~ACL_SQUITREMOTE;
+#endif
 #ifndef OPER_CONNECT
 	tmp &= ~ACL_CONNECT;
 #endif
@@ -335,6 +351,9 @@ long	oline_flags_parse(char *string)
 #endif
 #ifndef OPER_TKLINE
 	tmp &= ~ACL_TKLINE;
+#endif
+#ifndef OPER_KLINE
+	tmp &= ~ACL_KLINE;
 #endif
 	return tmp;
 }
@@ -625,6 +644,56 @@ aConfItem	*count_cnlines(Link *lp)
 	return nline;
 }
 
+#ifdef ENABLE_CIDR_LIMITS
+static int	add_cidr_limit(aClient *cptr, aConfItem *aconf)
+{
+	patricia_node_t *pnode;
+
+	if(aconf->class->cidr_amount == 0 || aconf->class->cidr_len == 0)
+		return -1;
+
+	pnode = patricia_match_ip(ConfCidrTree(aconf), &cptr->ip);
+
+	/* doesnt exist, create and then allow */
+	if(pnode == NULL)
+	{
+		pnode = patricia_make_and_lookup_ip(ConfCidrTree(aconf),
+					&cptr->ip,
+					aconf->class->cidr_len);
+
+		if(pnode == NULL)
+			return -1;
+
+		pnode->data++;
+		return 1;
+	}
+
+	if((long)pnode->data >= aconf->class->cidr_amount)
+		return 0;
+
+	pnode->data++;
+	return 1;
+}
+
+static void	remove_cidr_limit(aClient *cptr, aConfItem *aconf)
+{
+	patricia_node_t *pnode;
+
+	if(ConfMaxCidrAmount(aconf) == 0 || ConfCidrLen(aconf) == 0)
+		return;
+
+	pnode = patricia_match_ip(ConfCidrTree(aconf), &cptr->ip);
+
+	if(pnode == NULL)
+		return;
+
+	pnode->data--;
+
+	if(((unsigned long) pnode->data) == 0)
+		patricia_remove(ConfCidrTree(aconf), pnode);
+}
+#endif /* ENABLE_CIDR_LIMITS */
+
 /*
 ** detach_conf
 **	Disassociate configuration from the client.
@@ -644,8 +713,15 @@ int	detach_conf(aClient *cptr, aConfItem *aconf)
 			if ((aconf) && (Class(aconf)))
 			    {
 				if (aconf->status & CONF_CLIENT_MASK)
+				{
 					if (ConfLinks(aconf) > 0)
 						--ConfLinks(aconf);
+#ifdef ENABLE_CIDR_LIMITS
+					if ((aconf->status & CONF_CLIENT))
+						remove_cidr_limit(cptr, aconf);
+#endif
+				}
+
        				if (ConfMaxLinks(aconf) == -1 &&
 				    ConfLinks(aconf) == 0)
 		 		    {
@@ -711,8 +787,13 @@ int	attach_conf(aClient *cptr, aConfItem *aconf)
 		return -1; /* EXITC_FAILURE, hmm */
 	if ((aconf->status & (CONF_OPERATOR | CONF_CLIENT )))
 	    {
-		if (aconf->clients >= ConfMaxLinks(aconf) &&
-		    ConfMaxLinks(aconf) > 0)
+		if (
+#ifdef YLINE_LIMITS_OLD_BEHAVIOUR
+			aconf->clients >= ConfMaxLinks(aconf)
+#else
+			ConfLinks(aconf) >= ConfMaxLinks(aconf)
+#endif
+			&& ConfMaxLinks(aconf) > 0)
 			return -3;    /* EXITC_YLINEMAX */
 	    }
 
@@ -723,6 +804,10 @@ int	attach_conf(aClient *cptr, aConfItem *aconf)
 		anUser *user = NULL;
 		/* check on local/global limits per host and per user@host */
 
+#ifdef ENABLE_CIDR_LIMITS
+		if(!add_cidr_limit(cptr, aconf))
+			return -4; /* EXITC_LHMAX */
+#endif
 		/*
 		** local limits first to save CPU if any is hit.
 		**	host check is done on the IP address.
@@ -731,15 +816,30 @@ int	attach_conf(aClient *cptr, aConfItem *aconf)
 		if (ConfMaxHLocal(aconf) > 0 || ConfMaxUHLocal(aconf) > 0 ||
 		    ConfMaxHGlobal(aconf) > 0 || ConfMaxUHGlobal(aconf) > 0 )
 		{
+#ifdef YLINE_LIMITS_IPHASH
+			for ((user = hash_find_ip(cptr->user->sip, NULL));
+			     user; user = user->iphnext)
+				if (!mycmp(cptr->user->sip, user->sip))
+#else
 			for ((user = hash_find_hostname(cptr->sockhost, NULL));
 			     user; user = user->hhnext)
-			{
 				if (!mycmp(cptr->sockhost, user->host))
+#endif
 				{
 					ghcnt++;
+					if (ConfMaxHGlobal(aconf) > 0 &&
+					     ghcnt >= ConfMaxHGlobal(aconf))
+					{
+						return -6; /* EXITC_GHMAX */
+					}
 					if (MyConnect(user->bcptr))
 					{
 						hcnt++;
+						if (ConfMaxHLocal(aconf) > 0 &&
+						    hcnt >= ConfMaxHLocal(aconf))
+						{
+							return -4; /* EXITC_LHMAX */
+						}
 						if (!mycmp(user->bcptr->auth,
 							   cptr->auth))
 						{
@@ -762,23 +862,12 @@ int	attach_conf(aClient *cptr, aConfItem *aconf)
 						return -5; /* EXITC_LUHMAX */
 					}
 	
-					if (ConfMaxHLocal(aconf) > 0 &&
-					    hcnt >= ConfMaxHLocal(aconf))
-					{
-						return -4; /* EXITC_LHMAX */
-					}
 					if (ConfMaxUHGlobal(aconf) > 0 &&
 					    gucnt >= ConfMaxUHGlobal(aconf))
 					{
 						return -7; /* EXITC_GUHMAX */
 					}
-					if (ConfMaxHGlobal(aconf) > 0 &&
-					     ghcnt >= ConfMaxHGlobal(aconf))
-					{
-						return -6; /* EXITC_GHMAX */
-					}
 				}
-			}
 		}
 	}
 
@@ -924,20 +1013,12 @@ aConfItem	*find_conf_exact(char *name, char *user, char *host,
  */
 aConfItem	*find_Oline(char *name, aClient *cptr)
 {
-	Reg	aConfItem *tmp;
+	Reg	aConfItem *tmp, *tmp2 = NULL;
 	char	userhost[USERLEN+HOSTLEN+3];
 	char	userip[USERLEN+HOSTLEN+3];
 
 	sprintf(userhost, "%s@%s", cptr->username, cptr->sockhost);
-	sprintf(userip, "%s@%s", cptr->username, 
-#ifdef INET6
-		(char *)inetntop(AF_INET6, (char *)&cptr->ip, mydummy,
-			MYDUMMY_SIZE)
-#else
-		(char *)inetntoa((char *)&cptr->ip)
-#endif
-	);
-
+	sprintf(userip, "%s@%s", cptr->username, cptr->user->sip);
 
 	for (tmp = conf; tmp; tmp = tmp->next)
 	    {
@@ -954,8 +1035,10 @@ aConfItem	*find_Oline(char *name, aClient *cptr)
 			continue;
 		if (tmp->clients < MaxLinks(Class(tmp)))
 			return tmp;
+		else
+			tmp2 = tmp;
 	    }
-	return NULL;
+	return tmp2;
 }
 
 
@@ -1185,8 +1268,14 @@ int	rehash(aClient *cptr, aClient *sptr, int sig)
 	for (cltmp = NextClass(FirstClass()); cltmp; cltmp = NextClass(cltmp))
 		MaxLinks(cltmp) = -1;
 
+	if (sig == 'a')
+		start_iauth(2);	/* 2 means kill iauth first */
 	if (sig == 'd')
+	{
 		flush_cache();
+		close(resfd);
+		resfd = init_resolver(0x1f);
+	}
 #ifdef TKLINE
 	if (sig == 't')
 		tkline_expire(1);
@@ -1233,6 +1322,14 @@ int	openconf(void)
 {
 #ifdef	M4_PREPROC
 	int	pi[2], i;
+# ifdef HAVE_GNU_M4
+	char	*includedir, *includedirptr;
+
+	includedir = strdup(IRCDM4_PATH);
+	includedirptr = strrchr(includedir, '/');
+	if (includedirptr)
+		*includedirptr = '\0';
+# endif
 #else
 	int ret;
 #endif
@@ -1271,7 +1368,17 @@ int	openconf(void)
 		 * goes out with report_error.  Could be dangerous,
 		 * two servers running with the same fd's >:-) -avalon
 		 */
-		(void)execlp(M4_PATH, "m4", IRCDM4_PATH, configfile, 0);
+		(void)execlp(M4_PATH, "m4",
+#ifdef HAVE_GNU_M4
+#ifdef USE_M4_PREFIXES
+			"-P",
+#endif
+			"-I", includedir,
+#endif
+#ifdef INET6
+			"-DINET6",
+#endif
+			IRCDM4_PATH, configfile, (char *) NULL);
 		if (serverbooting)
 		{
 			fprintf(stderr,"Fatal Error: Error executing m4 (%s)",
@@ -1310,7 +1417,6 @@ char	*ipv6_convert(char *orig)
 	int	i, j;
 	int	len = 1;	/* for the '\0' in case of no @ */
 	struct	in6_addr addr;
-	char	dummy[MYDUMMY_SIZE];
 
 	if ((s = strchr(orig, '@')))
 	    {
@@ -1334,7 +1440,7 @@ char	*ipv6_convert(char *orig)
 
 	if (i > 0)
 	    {
-		t = inetntop(AF_INET6, addr.s6_addr, dummy, MYDUMMY_SIZE);
+		t = inetntop(AF_INET6, addr.s6_addr, ipv6string, sizeof(ipv6string));
 	    }
 	
 	j = len - 1;
@@ -1377,6 +1483,9 @@ int 	initconf(int opt)
 	Reg	char	*tmp, *s;
 	int	fd, i;
 	char	*tmp2 = NULL, *tmp3 = NULL, *tmp4 = NULL;
+#ifdef ENABLE_CIDR_LIMITS
+	char	*tmp5 = NULL;
+#endif
 	int	ccount = 0, ncount = 0;
 	aConfItem *aconf = NULL;
 #if defined(CONFIG_DIRECTIVE_INCLUDE)
@@ -1397,6 +1506,16 @@ int 	initconf(int opt)
 	    }
 #if defined(CONFIG_DIRECTIVE_INCLUDE)
 	fdn = fdopen(fd, "r");
+	if (fdn == NULL)
+	{
+		if (serverbooting)
+		{
+			fprintf(stderr,
+			"Fatal Error: Can not open configuration file %s (%s)\n",
+			configfile,strerror(errno));
+		}
+		return -1;
+	}
 	ConfigTop = config_read(fdn, 0, new_config_file(configfile, NULL, 0));
 	for(p = ConfigTop; p; p = p->next)
 #else
@@ -1460,6 +1579,9 @@ int 	initconf(int opt)
 		if (tmp2)
 			MyFree(tmp2);
 		tmp3 = tmp4 = NULL;
+#ifdef ENABLE_CIDR_LIMITS
+		tmp5 = NULL;
+#endif
 		tmp = getfield(line);
 		if (!tmp)
 			continue;
@@ -1583,8 +1705,13 @@ int 	initconf(int opt)
 #ifdef XLINE
 			if (aconf->status == CONF_XLINE)
 			{
+				DupString(aconf->name2, tmp);
+				if ((tmp = getfield(NULL)) == NULL)
+					break;
+				DupString(aconf->name3, tmp);
+				if ((tmp = getfield(NULL)) == NULL)
+					break;
 				DupString(aconf->source_ip, tmp);
-				/* For X: we stop parsing after 4th field */
 				break;
 			}
 #endif
@@ -1603,11 +1730,20 @@ int 	initconf(int opt)
 			if ((tmp3 = getfield(NULL)) == NULL)
 				break;
 			/* used in Y: global limits */
-			tmp4 = getfield(NULL);
+			if((tmp4 = getfield(NULL)) == NULL)
+				break;
+#ifdef ENABLE_CIDR_LIMITS
+			tmp5 = getfield(NULL);
+#endif
 		} while (0); /* to use break without compiler warnings */
 		istat.is_confmem += aconf->host ? strlen(aconf->host)+1 : 0;
 		istat.is_confmem += aconf->passwd ? strlen(aconf->passwd)+1 :0;
 		istat.is_confmem += aconf->name ? strlen(aconf->name)+1 : 0;
+		istat.is_confmem += aconf->name2 ? strlen(aconf->name2)+1 : 0;
+#ifdef XLINE
+		istat.is_confmem += aconf->name3 ? strlen(aconf->name3)+1 : 0;
+#endif
+		istat.is_confmem += aconf->source_ip ? strlen(aconf->source_ip)+1 : 0;
 
 		/*
 		** Bounce line fields are mandatory
@@ -1625,12 +1761,18 @@ int 	initconf(int opt)
 					  atoi(aconf->passwd),
 					  atoi(aconf->name), aconf->port,
 					  tmp ? atoi(tmp) : 0,
+					  (tmp && index(tmp, '.')) ?
+					  atoi(index(tmp, '.') + 1) : 0,
 					  tmp3 ? atoi(tmp3) : 1,
 					  (tmp3 && index(tmp3, '.')) ?
 					  atoi(index(tmp3, '.') + 1) : 1,
  					  tmp4 ? atoi(tmp4) : 1,
 					  (tmp4 && index(tmp4, '.')) ?
-					  atoi(index(tmp4, '.') + 1) : 1);
+					  atoi(index(tmp4, '.') + 1) : 1
+#ifdef ENABLE_CIDR_LIMITS
+					  , tmp5
+#endif
+				);
 			continue;
 		}
 		/*
@@ -1659,7 +1801,11 @@ int 	initconf(int opt)
 
 			/* trying to find exact conf line in already existing
 			 * conf, so we don't delete old one, just update it */
-			if ((bconf = find_conf_entry(aconf, aconf->status)))
+			if (
+#ifdef FASTER_ILINE_REHASH
+				(aconf->status & CONF_LISTEN_PORT) &&
+#endif
+				(bconf = find_conf_entry(aconf, aconf->status)))
 			{
 				/* we remove bconf (already existing) from conf
 				 * so that we can add it back uniformly at the
@@ -1694,10 +1840,22 @@ int 	initconf(int opt)
 		if (aconf->status & CONF_SERVICE)
 			aconf->port &= SERVICE_MASK_ALL;
 		if (aconf->status & (CONF_SERVER_MASK|CONF_SERVICE))
-			if (ncount > MAXCONFLINKS || ccount > MAXCONFLINKS ||
-			    !aconf->host || index(aconf->host, '*') ||
-			     index(aconf->host,'?') || !aconf->name)
-				continue;
+		{
+			char *hostptr = NULL;
+
+			/* since it's u@h syntax, let's ignore user part
+			   in checks below --B. */
+			hostptr = index(aconf->host, '@');
+			if (hostptr != NULL)
+				hostptr++;	/* move ptr after '@' */
+			else
+				hostptr = aconf->host;
+
+			if (ncount > MAXCONFLINKS || ccount > MAXCONFLINKS
+				|| !hostptr || index(hostptr, '*')
+				|| index(hostptr,'?') || !aconf->name)
+				continue;	/* next config line */
+		}
 
 		if (aconf->status &
 		    (CONF_SERVER_MASK|CONF_OPERATOR|CONF_SERVICE))
@@ -1913,8 +2071,8 @@ int	find_kill(aClient *cptr, int timedklines, char **comment)
 
 	host = cptr->sockhost;
 #ifdef INET6
-	ip = (char *) inetntop(AF_INET6, (char *)&cptr->ip, mydummy,
-			       MYDUMMY_SIZE);
+	ip = (char *) inetntop(AF_INET6, (char *)&cptr->ip, ipv6string,
+			       sizeof(ipv6string));
 #else
 	ip = (char *) inetntoa((char *)&cptr->ip);
 #endif
@@ -2298,9 +2456,11 @@ aConfItem	*find_denied(char *name, int class)
  * Parses 0w1d2h3m4s timeformat, filling in output variable in seconds.
  * Returns 0 if everything went ok.
  */
-int	wdhms2sec(char *input, int *output)
+int	wdhms2sec(char *input, time_t *output)
 {
-#define DEFAULT_MULTI 60
+#ifndef TKLINE_MULTIPLIER
+#define TKLINE_MULTIPLIER 60
+#endif
 	int multi;
 	int tmp = 0;
 	char *s;
@@ -2312,7 +2472,7 @@ int	wdhms2sec(char *input, int *output)
 	s = input;
 	while (*s)
 	{
-		switch(*s)
+		switch(tolower(*s))
 		{
 		case 'w':
 			multi = 604800; break;
@@ -2332,7 +2492,7 @@ int	wdhms2sec(char *input, int *output)
 					s++;
 				if (!*s)
 				{
-					*output += DEFAULT_MULTI * tmp;
+					*output += TKLINE_MULTIPLIER * tmp;
 				}
 				continue;
 			}
@@ -2344,66 +2504,16 @@ int	wdhms2sec(char *input, int *output)
 	}
 	return 0;
 }
-
-int	m_tkline(aClient *cptr, aClient *sptr, int parc, char **parv)
-{
-	int	time, status = CONF_TKILL;
-	char	*user, *host, *reason;
-	int	i;
-
-	if (!is_allowed(sptr, ACL_TKLINE))
-		return m_nopriv(cptr, sptr, parc, parv);
-
-	/* sanity checks */
-	i = wdhms2sec(parv[1], &time);
-	user = parv[2];
-	host = strchr(user, '@');
-	if (i || !host)
-	{
-		/* error */
-		if (!IsPerson(sptr))
-		{
-			sendto_one(sptr, ":%s NOTICE %s "
-				":TKLINE: Incorrect format",
-				ME, parv[0]);
-			return exit_client(cptr, cptr, &me,
-				"TKLINE: Incorrect format");
-		}
-		sendto_one(sptr, ":%s NOTICE %s :TKLINE: Incorrect format",
-			ME, parv[0]);
-		return 2;
-	}
-
-	/* All seems fine. */
-#ifdef TKLINE_MAXTIME
-	if (time > TKLINE_MAXTIME)
-		time = TKLINE_MAXTIME;
 #endif
-	if (*user == '=')
-	{
-		status = CONF_TOTHERKILL;
-		user++;
-	}
-	*host++ = '\0';
-	reason = parv[3];
-	if (strlen(reason) > TOPICLEN)
-	{
-		reason[TOPICLEN] = '\0';
-	}
-
-	/* All parameters are now sane. Do the stuff. */
-	do_tkline(parv[0], time, user, host, reason, status);
-
-	return 1;
-}
 
+#if defined(TKLINE) || defined(KLINE)
 /* 
- * Adds tkline to tkconf.
+ * Adds t/kline to t/kconf.
  * If tkline already existed, its expire time is updated.
  *
  * Returns created tkline expire time.
  */
-void do_tkline(char *who, int time, char *user, char *host, char *reason, int status)
+void do_kline(int tkline, char *who, time_t time, char *user, char *host, char *reason, int status)
 {
 	char buff[BUFSIZE];
 	aClient	*acptr;
@@ -2413,7 +2523,7 @@ void do_tkline(char *who, int time, char *user, char *host, char *reason, int st
 	buff[0] = '\0';
 
 	/* Check if such u@h already exists in tkconf. */
-	for (aconf = tkconf; aconf; aconf = aconf->next)
+	for (aconf = tkline?tkconf:kconf; aconf; aconf = aconf->next)
 	{
 		if (0==strcasecmp(aconf->host, host) && 
 			0==strcasecmp(aconf->name, user))
@@ -2437,17 +2547,30 @@ void do_tkline(char *who, int time, char *user, char *host, char *reason, int st
 		istat.is_confmem += strlen(aconf->host) + 1;
 		istat.is_confmem += strlen(aconf->passwd) + 1;
 
-		/* put on top of tkconf */
-		if (tkconf)
+		/* put on top of t/kconf */
+		if (tkline)
 		{
-			aconf->next = tkconf;
+			if (tkconf)
+			{
+				aconf->next = tkconf;
+			}
+			tkconf = aconf;
+			sendto_flag(SCH_TKILL, "TKLINE %s@%s (%u) by %s :%s",
+				aconf->name, aconf->host, time, who, reason);
+		}
+		else
+		{
+			if (kconf)
+			{
+				aconf->next = kconf;
+			}
+			kconf = aconf;
+			sendto_flag(SCH_TKILL, "KLINE %s@%s by %s :%s",
+				aconf->name, aconf->host, who, reason);
 		}
-		tkconf = aconf;
 	}
-	sendto_flag(SCH_TKILL, "TKLINE %s@%s (%d) by %s :%s",
-		aconf->name, aconf->host, time, who, reason);
 
-	/* get rid of tklined clients */
+	/* get rid of klined clients */
 	for (i = highest_fd; i >= 0; i--)
 	{
 		if (!(acptr = local[i]) || !IsPerson(acptr)
@@ -2513,29 +2636,190 @@ void do_tkline(char *who, int time, char *user, char *host, char *reason, int st
 				ME, acptr->name, aconf->name, aconf->host,
 				": ", aconf->passwd);
 			sendto_flag(SCH_TKILL,
-				"TKill line active for %s",
+				"%sKill line active for %s", tkline?"T":"",
 				get_client_name(acptr, FALSE));
 			if (buff[0] == '\0')
 			{
 				sprintf(buff, "Kill line active: %.80s",
 					aconf->passwd);
 			}
+			acptr->exitc = tkline ? EXITC_TKLINE : EXITC_KLINE;
 			(void) exit_client(acptr, acptr, &me, buff);
 		}
 	}
 	if (count > 4)
 	{
-		sendto_flag(SCH_TKILL, "TKill reaped %d souls", count);
+		sendto_flag(SCH_TKILL, "%sKline reaped %d souls",
+			tkline?"T":"", count);
 	}
 
+#ifdef TKLINE
 	/* do next tkexpire, but not more often than once a minute */
 	if (!nexttkexpire || nexttkexpire > aconf->hold)
 	{
 		nexttkexpire = MAX(timeofday + 60, aconf->hold);
 	}
+#endif
 	return;
 }
 
+int	prep_kline(int tkline, aClient *cptr, aClient *sptr, int parc, char **parv)
+{
+	int	status = tkline ? CONF_TKILL : CONF_KILL;
+	time_t	time;
+	char	*user, *host, *reason;
+	int	err = 0;
+
+	/* sanity checks */
+	if (tkline)
+	{
+		err = wdhms2sec(parv[1], &time);
+#ifdef TKLINE_MAXTIME
+		if (time > TKLINE_MAXTIME)
+			time = TKLINE_MAXTIME;
+		if (time < 0) /* overflown, must have wanted bignum :) */
+			time = TKLINE_MAXTIME;
+#endif
+		user = parv[2];
+		reason = parv[3];
+	}
+	else
+	{
+		user = parv[1];
+		reason = parv[2];
+	}
+	host = strchr(user, '@');
+	
+	if (strlen(user) > USERLEN+HOSTLEN+1)
+	{
+		err = 1;
+	}
+	if (host)
+	{
+		*host++ = '\0';
+	}
+	if (!user || !host || *user == '\0' || *host == '\0' ||
+		(!strcmp("*", user) && !strcmp("*", host))) {
+		/* disallow all forms of bad u@h format and block *@* too */
+		err = 1;
+	}
+	if (!err && host && strchr(host, '/') && match_ipmask(host, sptr, 0) == -1)
+	{
+		/* check validity of 1.2.3.0/24 or it will be spewing errors
+		** for every connecting client. */
+		err = 1;
+	}
+#ifdef KLINE
+badkline:
+#endif
+	if (err)
+	{
+		/* error */
+		if (!IsPerson(sptr))
+		{
+			sendto_one(sptr, ":%s NOTICE %s "
+				":T/KLINE: Incorrect format",
+				ME, parv[0]);
+			return exit_client(cptr, cptr, &me,
+				"T/KLINE: Incorrect format");
+		}
+		sendto_one(sptr, ":%s NOTICE %s :%sKLINE: Incorrect format",
+			ME, parv[0], tkline?"T":"");
+		return 2;
+	}
+
+	/* All seems fine. */
+	if (*user == '=')
+	{
+		status = tkline ? CONF_TOTHERKILL : CONF_OTHERKILL;
+		user++;
+	}
+#ifdef INET6
+	host = ipv6_convert(host);
+#endif
+	if (strlen(reason) > TOPICLEN)
+	{
+		reason[TOPICLEN] = '\0';
+	}
+
+#ifdef KLINE
+	if (!tkline)
+	{
+		int	kfd, ksize, kret;
+		char	kbuf[2*BUFSIZE];
+		char	*utmp, *htmp, *rtmp;
+
+		if (!strcmp(KLINE_PATH, IRCDCONF_PATH))
+		{
+			sendto_flag(SCH_ERROR,
+				"Invalid kline configuration file.");
+			return MAXPENALTY;
+		}
+		utmp = strchr(user, IRCDCONF_DELIMITER);
+		htmp = strchr(host, IRCDCONF_DELIMITER);
+		rtmp = strchr(reason, IRCDCONF_DELIMITER);
+		if (utmp || htmp || rtmp)
+		{
+			/* Too lazy to copy it here. --B. */
+			err = 1;
+			goto badkline;
+		}
+
+		kfd = open(KLINE_PATH, O_WRONLY|O_APPEND|O_NDELAY);
+		if (kfd < 0)
+		{
+			sendto_flag(SCH_ERROR,
+				"Cannot open kline configuration file.");
+			return MAXPENALTY;
+		}
+		ksize = snprintf(kbuf, sizeof(kbuf),
+			"%c%c%s%c%s%c%s%c0%c #%s%s%s%s%s#%d\n",
+			(status == CONF_OTHERKILL ? 'k' : 'K'),
+			IRCDCONF_DELIMITER, host, IRCDCONF_DELIMITER, reason,
+			IRCDCONF_DELIMITER, user, IRCDCONF_DELIMITER,
+			IRCDCONF_DELIMITER, sptr->name,
+			sptr->user ? "!" : "",
+			sptr->user ? sptr->user->username : "",
+			sptr->user ? "@" : "",
+			sptr->user ? sptr->user->host : "", (int)timeofday);
+		kret = write(kfd, kbuf, ksize);
+		close(kfd);
+		if (kret != ksize)
+		{
+			sendto_flag(SCH_ERROR, "Error writing (%d!=%d) "
+				"to kline configuration file.", kret, ksize);
+			sendto_one(sptr, ":%s NOTICE %s :KLINE: error writing "
+				"(%d!=%d) to kline configuration file",
+				ME, parv[0], kret, ksize);
+			return MAXPENALTY;
+		}
+	}
+#endif /* KLINE */
+
+	/* All parameters are now sane. Do the stuff. */
+	do_kline(tkline, parv[0], time, user, host, reason, status);
+
+	return 1;
+}
+#endif /* TKLINE || KLINE */
+
+#ifdef KLINE
+int	m_kline(aClient *cptr, aClient *sptr, int parc, char **parv)
+{
+	if (!is_allowed(sptr, ACL_KLINE))
+		return m_nopriv(cptr, sptr, parc, parv);
+	return prep_kline(0, cptr, sptr, parc, parv);
+}
+#endif
+
+#ifdef TKLINE
+int	m_tkline(aClient *cptr, aClient *sptr, int parc, char **parv)
+{
+	if (!is_allowed(sptr, ACL_TKLINE))
+		return m_nopriv(cptr, sptr, parc, parv);
+	return prep_kline(1, cptr, sptr, parc, parv);
+}
+
 int	m_untkline(aClient *cptr, aClient *sptr, int parc, char **parv)
 {
 	aConfItem	*tmp, *prev;
@@ -2619,4 +2903,4 @@ time_t	tkline_expire(int all)
 		min = nexttkexpire + 60;
 	return min;
 }
-#endif
+#endif /* TKLINE */
