@@ -19,7 +19,7 @@
  */
 
 #ifndef lint
-static const volatile char rcsid[] = "@(#)$Id: s_debug.c,v 1.47 2004/10/01 20:22:15 chopin Exp $";
+static const volatile char rcsid[] = "@(#)$Id: s_debug.c,v 1.56 2009/11/13 20:25:55 chopin Exp $";
 #endif
 
 #include "os.h"
@@ -63,9 +63,12 @@ char	serveropts[] = {
 #ifdef	BETTER_CDELAY
 'h',
 #endif
-#ifndef	NO_DEFAULT_INVISIBLE
+#ifdef	DEFAULT_INVISIBLE
 'I',
 #endif
+#ifdef	JAPANESE
+'j',
+#endif
 #ifdef	OPER_DIE
 'J',
 #endif
@@ -97,6 +100,9 @@ char	serveropts[] = {
 #ifdef	CRYPT_LINK_PASSWORD
 'P',
 #endif
+#ifdef	OPER_SQUIT_REMOTE
+'Q',
+#endif
 #ifdef	OPER_RESTART
 'R',
 #endif
@@ -112,9 +118,12 @@ char	serveropts[] = {
 #ifndef	NO_PREFIX
 'u',
 #endif
-#ifdef	ENABLE_USERS
+#ifdef	USERS_SHOWS_UTMP
 'U',
 #endif
+#ifdef	ENABLE_SIDTRACE
+'v',
+#endif
 #ifdef	UNIXPORT
 'X',
 #endif
@@ -275,7 +284,7 @@ void	send_usage(aClient *cptr, char *nick)
 	return;
 }
 
-void	send_defines(aClient *cptr, char *nick)
+void	send_defines(aClient *cptr, char *nick, char *extend)
 {
     	sendto_one(cptr, ":%s %d %s :HUB:%s MS:%d", 
 		   ME, RPL_STATSDEFINE, nick,
@@ -294,8 +303,14 @@ void	send_defines(aClient *cptr, char *nick)
 		   ME, RPL_STATSDEFINE, nick, KILLCHASETIMELIMIT,
 		   DELAYCHASETIMELIMIT, LDELAYCHASETIMELIMIT,
 		   CLIENT_FLOOD, MAXCHANNELSPERUSER);
-	sendto_one(cptr, ":%s %d %s :H:%d N:%d D:%d U:%d R:%d T:%d C:%d P:%d K:%d",
-		   ME, RPL_STATSDEFINE, nick, HOSTLEN, ONICKLEN, UIDLEN, USERLEN,
+	sendto_one(cptr, ":%s %d %s :H:%d N:%d N0:%d D:%d U:%d R:%d T:%d C:%d P:%d K:%d",
+		   ME, RPL_STATSDEFINE, nick, HOSTLEN, LOCALNICKLEN, 
+#ifdef MINLOCALNICKLEN
+		   MINLOCALNICKLEN,
+#else
+		   1,
+#endif
+		   UIDLEN, USERLEN,
 		   REALLEN, TOPICLEN, CHANNELLEN, PASSWDLEN, KEYLEN);
 	sendto_one(cptr, ":%s %d %s :BS:%d MXR:%d MXB:%d MXBL:%d PY:%d",
 		   ME, RPL_STATSDEFINE, nick, BUFSIZE, MAXRECIPIENTS, MAXBANS,
@@ -318,13 +333,25 @@ void	send_defines(aClient *cptr, char *nick)
 		-1
 #endif
 		);
-	sendto_one(cptr, ":%s %d %s :AC:%d CA:%d S:%d SS:%d SU:%d", ME, RPL_STATSDEFINE,
-		   nick, iconf.aconnect, iconf.caccept, iconf.split,
-		   iconf.split_minservers, iconf.split_minusers);
+	sendto_one(cptr, ":%s %d %s :AC:%d CA:%d S:%d SS:%d/%d/%d SU:%d/%d/%d",
+		ME, RPL_STATSDEFINE, nick, iconf.aconnect, iconf.caccept, 
+		iconf.split, SPLIT_SERVERS, iconf.split_minservers, istat.is_eobservers,
+		SPLIT_USERS, iconf.split_minusers, istat.is_user[0] + istat.is_user[1]);
 #ifdef CLIENTS_CHANNEL
 	sendto_one(cptr, ":%s %d %s :CCL:0x%X", ME, RPL_STATSDEFINE, nick,
 		CLIENTS_CHANNEL_LEVEL);
 #endif
+	/* note that it's safe to check extend[1], it will at worst be null.
+	** if we ever need extend[2], check length first... --B. */
+	if (extend[1] == '5')
+	{
+		char **isup = isupport;
+		while (*isup)
+		{
+      	sendto_one(cptr, replies[RPL_ISUPPORT], ME, nick, *isup);
+			isup++;
+		}
+	}
 }
 
 void	count_memory(aClient *cptr, char *nick, int debug)
