@@ -22,7 +22,7 @@
  */
 
 #ifndef lint
-static const volatile char rcsid[] = "@(#)$Id: s_misc.c,v 1.103 2005/03/28 23:33:27 chopin Exp $";
+static const volatile char rcsid[] = "@(#)$Id: s_misc.c,v 1.119 2009/11/13 20:08:11 chopin Exp $";
 #endif
 
 #include "os.h"
@@ -200,7 +200,7 @@ char	*get_client_name(aClient *sptr, int showip)
 #ifdef INET6 
 					      inetntop(AF_INET6,
 						       (char *)&sptr->ip,
-						       mydummy, MYDUMMY_SIZE)
+						       ipv6string, sizeof(ipv6string))
 #else
 					      inetntoa((char *)&sptr->ip)
 #endif
@@ -233,8 +233,8 @@ char	*get_client_host(aClient *cptr)
 
 	if (!MyConnect(cptr))
 		return cptr->name;
-	if (!cptr->hostp)
-		return get_client_name(cptr, FALSE);
+	if (!cptr->user)
+		return get_client_name(cptr, TRUE);
 #ifdef UNIXPORT
 	if (IsUnixSocket(cptr))
 		sprintf(nbuf, "%s[%s]", cptr->name, ME);
@@ -243,7 +243,7 @@ char	*get_client_host(aClient *cptr)
 		(void)sprintf(nbuf, "%s[%-.*s@%-.*s]",
 			cptr->name, USERLEN,
 			(!(cptr->flags & FLAGS_GOTID)) ? "" : cptr->auth,
-			HOSTLEN, cptr->hostp->h_name);
+			HOSTLEN, cptr->user->sip);
 	return nbuf;
 }
 
@@ -741,7 +741,7 @@ static	void	exit_one_client(aClient *cptr, aClient *sptr, aClient *from,
 			}
 		}
 #ifdef	USE_SERVICES
-		check_services_butone(SERVICE_WANT_SQUIT, sptr->name, sptr,
+		check_services_butone(SERVICE_WANT_SQUIT, sptr->serv, sptr,
 				      ":%s SQUIT %s :%s", from->name,
 				      sptr->name, comment);
 #endif
@@ -759,30 +759,30 @@ static	void	exit_one_client(aClient *cptr, aClient *sptr, aClient *from,
 		; /* Nothing */
 	}
 	else if (sptr->name[0] && !IsService(sptr)) /* clean with QUIT... */
-	    {
+	{
 		/*
 		** If this exit is generated from "m_kill", then there
 		** is no sense in sending the QUIT--KILL's have been
 		** sent instead.
 		*/
 		if ((sptr->flags & FLAGS_KILLED) == 0)
-		    {
+		{
 			if ((sptr->flags & FLAGS_SPLIT) == 0)
-			    {
+			{
 				sendto_serv_butone(cptr, ":%s QUIT :%s",
 						   sptr->user->uid, comment);
 #ifdef	USE_SERVICES
 				check_services_butone(SERVICE_WANT_QUIT|
 						      SERVICE_WANT_RQUIT, 
 						      (sptr->user) ?
-						      sptr->user->server
+						      sptr->user->servp
 						      : NULL, cptr,
 						      ":%s QUIT :%s",
 						      sptr->name, comment);
 #endif
-			    }
+			}
 			else
-			    {
+			{
 				if (sptr->flags & FLAGS_HIDDEN)
 					/* joys of hostmasking */
 					for (i = fdas.highest; i >= 0; i--)
@@ -799,13 +799,28 @@ static	void	exit_one_client(aClient *cptr, aClient *sptr, aClient *from,
 					}
 #ifdef	USE_SERVICES
 				check_services_butone(SERVICE_WANT_QUIT, 
-					      (sptr->user) ? sptr->user->server
+					      (sptr->user) ? sptr->user->servp
 						      : NULL, cptr,
 						      ":%s QUIT :%s",
 						      sptr->name, comment);
 #endif
-			    }
-		    }
+			}
+		}
+#ifdef USE_SERVICES
+		else
+		{
+			/* Send QUIT to services which desire such as well.
+			** Services with both _QUIT and _KILL will get both
+			** for now --jv
+			*/
+			check_services_butone(SERVICE_WANT_QUIT, 
+					     (sptr->user) ? sptr->user->servp
+						      : NULL, cptr,
+						      ":%s QUIT :%s",
+						      sptr->name, comment);
+
+		}
+#endif
 		/*
 		** If a person is on a channel, send a QUIT notice
 		** to every client (person) on the same channel (so
@@ -887,8 +902,13 @@ static	void	exit_one_client(aClient *cptr, aClient *sptr, aClient *from,
 			add_history(sptr, (sptr == cptr) ? &me : NULL);
 #endif
 			off_history(sptr);
+#ifdef USE_HOSTHASH
 			del_from_hostname_hash_table(sptr->user->host,
 						     sptr->user);
+#endif
+#ifdef USE_IPHASH
+			del_from_ip_hash_table(sptr->user->sip, sptr->user);
+#endif
 		    }
 	    }
 	else if (sptr->name[0] && IsService(sptr))
@@ -913,7 +933,8 @@ static	void	exit_one_client(aClient *cptr, aClient *sptr, aClient *from,
 				{
 					continue;
 				}
-				if (match(sptr->service->dist, acptr->name))
+				if (match(sptr->service->dist, acptr->name) && 
+					match(sptr->service->dist, acptr->serv->sid))
 				{
 					continue;
 				}
@@ -990,8 +1011,14 @@ void	initruntimeconf(void)
 	iconf.caccept = 2; /* accept clients when no split */
 
 	/* Defaults set in config.h */
-	iconf.split_minservers = SPLIT_SERVERS;
-	iconf.split_minusers = SPLIT_USERS;
+	iconf.split_minservers = DEFAULT_SPLIT_SERVERS;
+	iconf.split_minusers = DEFAULT_SPLIT_USERS;
+
+	if ((bootopt & BOOT_STANDALONE))
+	{
+		/* standalone mode */
+		iconf.split = 3;
+	}
 }
 
 void	tstats(aClient *cptr, char *name)
@@ -1065,6 +1092,8 @@ void	tstats(aClient *cptr, char *name)
 		ME, RPL_STATSDEBUG, name, istat.is_delayclosewait,
 		istat.is_delayclose);
 #endif
+	sendto_one(cptr, ":%s %d %s :local channels reops %d remote %d",
+		ME, RPL_STATSDEBUG, name, sp->is_reop, sp->is_rreop);
 	sendto_one(cptr, ":%s %d %s :Client - Server",
 		   ME, RPL_STATSDEBUG, name);
 	sendto_one(cptr, ":%s %d %s :connected %lu %lu",
@@ -1123,8 +1152,6 @@ void	read_motd(char *filename)
 		else
 			line[len] = '\0';
 		temp = (aMotd *)MyMalloc(sizeof(aMotd));
-		if (!temp)
-			outofmemory();
 		temp->line = mystrdup(line);
 		temp->next = NULL;
 		       if (!motd)
@@ -1139,6 +1166,9 @@ void	read_motd(char *filename)
 
 void	check_split(void)
 {
+	if ((bootopt & BOOT_STANDALONE))
+		return;
+
 	if (istat.is_eobservers < iconf.split_minservers ||
 	    istat.is_user[0] + istat.is_user[1] < iconf.split_minusers)
 	{
@@ -1147,7 +1177,7 @@ void	check_split(void)
 		{
 			sendto_flag(SCH_NOTICE,
 				"Network split detected, split mode activated");
-			iconf.split = 1;
+			iconf.split = timeofday;
 		}
 	}
 	else
@@ -1162,6 +1192,9 @@ void	check_split(void)
 			{
 				firstrejoindone = 1;
 				activate_delayed_listeners();
+#ifdef CACCEPT_DEFAULT
+				iconf.caccept = CACCEPT_DEFAULT;
+#endif
 			}
 		}
 	}
