@@ -22,7 +22,7 @@
  */
 
 #ifndef lint
-static const volatile char rcsid[] = "@(#)$Id: s_serv.c,v 1.274 2005/05/14 20:51:26 chopin Exp $";
+static const volatile char rcsid[] = "@(#)$Id: s_serv.c,v 1.296 2008/06/21 12:00:47 chopin Exp $";
 #endif
 
 #include "os.h"
@@ -40,7 +40,7 @@ static	void	report_listeners(aClient *, char *);
 static	void	count_servers_users(aClient *, int *, int *);
 const	char	*check_servername_errors[3][2] = {
 	{ "too long", "Bogus servername - too long" },
-	{ "invalid", "Bogus servername - invalid hostname" },
+	{ "invalid", "Bogus servername - invalid chars in hostname" },
 	{ "bogus", "Bogus servername - no dot"}};
 static	int	send_users(aClient *, aClient *, int, char **);
 
@@ -152,6 +152,10 @@ int	m_squit(aClient *cptr, aClient *sptr, int parc, char *parv[])
 	** when the command is issued by an oper.
 	*/
 	if (!acptr)
+	{
+		acptr = find_server(server, NULL);
+	}
+	if (!acptr)
 	{
 		for (acptr = client;
 			(acptr = next_client(acptr, server));
@@ -385,6 +389,11 @@ int	check_version(aClient *cptr)
 		}
 	}
 
+#ifdef JAPANESE
+	if (link && strchr(link, 'j'))	/* jp version */
+		cptr->flags |= FLAGS_JP;
+#endif
+
 	/* right now, I can't code anything good for this */
 	/* Stop whining, and do it! ;) */
 	if (link && strchr(link, 'Z'))	/* Compression requested */
@@ -567,6 +576,13 @@ int	m_server(aClient *cptr, aClient *sptr, int parc, char *parv[])
                 sendto_one(sptr, replies[ERR_ALREADYREGISTRED], ME, BadTo(parv[0]));
                 return 1;
             }
+	if ((bootopt & BOOT_STANDALONE))
+	{
+		sendto_flag(SCH_ERROR,
+			"Running in standalone mode, cannot accept %s/%s", parv[1], parv[3]);
+		return exit_client(cptr, cptr, &me,
+			"Running in standalone mode");
+	}
 	info[0] = info[REALLEN] = '\0';	/* strncpy() doesn't guarantee NULL */
 	inpath = get_client_name(cptr, FALSE);
 	if (parc < 2 || *parv[1] == '\0')
@@ -786,7 +802,7 @@ int	m_server(aClient *cptr, aClient *sptr, int parc, char *parv[])
 
 		introduce_server(cptr, acptr);
 #ifdef	USE_SERVICES
-		check_services_butone(SERVICE_WANT_SERVER, acptr->name, acptr,
+		check_services_butone(SERVICE_WANT_SERVER, acptr->serv, acptr,
 				      ":%s SERVER %s %d %s :%s", parv[0],
 				      acptr->name, hop+1, acptr->serv->sid,
 				      acptr->info);
@@ -1026,16 +1042,19 @@ int	m_server_estab(aClient *cptr, char *sid, char *versionbuf)
 		    }
 
 		if (bconf->passwd[0])
-#ifndef	ZIP_LINKS
-			sendto_one(cptr, "PASS %s %s IRC|%s %s", bconf->passwd,
-				   pass_version, serveropts,
-				   (bootopt & BOOT_STRICTPROT) ? "P" : "");
+			sendto_one(cptr, "PASS %s %s IRC|%s %s%s%s",
+				bconf->passwd, pass_version, serveropts,
+				(bootopt & BOOT_STRICTPROT) ? "P" : "",
+#ifdef	JAPANESE
+				"j",
 #else
-			sendto_one(cptr, "PASS %s %s IRC|%s %s%s",
-				   bconf->passwd, pass_version, serveropts,
-			   (bconf->status == CONF_ZCONNECT_SERVER) ? "Z" : "",
-				   (bootopt & BOOT_STRICTPROT) ? "P" : "");
+				"",
 #endif
+#ifdef	ZIP_LINKS
+				(bconf->status == CONF_ZCONNECT_SERVER) ? "Z" :
+#endif
+				""
+				);
 		/*
 		** Pass my info to the new server
 		*/
@@ -1198,12 +1217,12 @@ int	m_server_estab(aClient *cptr, char *sid, char *versionbuf)
 		cptr->name, cptr->serv->version, cptr->serv->sid));
 	add_fd(cptr->fd, &fdas);
 #ifdef	USE_SERVICES
-	check_services_butone(SERVICE_WANT_SERVER, cptr->name, cptr,
+	check_services_butone(SERVICE_WANT_SERVER, cptr->serv, cptr,
 			      ":%s SERVER %s %d %s :%s", ME, cptr->name,
 			      cptr->hopcount+1, cptr->serv->sid, cptr->info);
 #endif
-	sendto_flag(SCH_SERVER, "Received SERVER %s (%d %s)", cptr->name,
-		    1, cptr->info);
+	sendto_flag(SCH_SERVER, "Received SERVER %s from %s (%d %s)",
+		cptr->name, ME, 1, cptr->info);
 	introduce_server(cptr, cptr);
 	
 	/*
@@ -1249,7 +1268,8 @@ int	m_server_estab(aClient *cptr, char *sid, char *versionbuf)
 					   (*buf) ? buf : "+", acptr->info);
 		    }
 		else if (IsService(acptr) &&
-			 match(acptr->service->dist, cptr->name) == 0)
+			 (match(acptr->service->dist, cptr->name) == 0 ||
+				match(acptr->service->dist, cptr->serv->sid) == 0))
 		{
 			sendto_one(cptr, ":%s SERVICE %s %s %d :%s",
 						acptr->service->servp->sid,
@@ -1699,10 +1719,11 @@ static  void    report_x_lines(aClient *sptr, char *to)
 		if (tmp->status != CONF_XLINE)
 			continue;
 
-		sendto_one(sptr,":%s %d %s X :%s %s %s %s", 
+		sendto_one(sptr,":%s %d %s X :%s %s %s %s %s %s", 
 			ME, RPL_STATSDEBUG, to,
 			BadTo(tmp->host), BadTo(tmp->passwd),
-			BadTo(tmp->name), BadTo(tmp->source_ip));
+			BadTo(tmp->name), BadTo(tmp->name2),
+			BadTo(tmp->name3), BadTo(tmp->source_ip));
 	}
 }
 #endif
@@ -1816,7 +1837,7 @@ static void report_fd(aClient *sptr, aClient *acptr, char *to)
 #ifdef INET6
 		inetntop(AF_INET6,
 		(char *)&acptr->acpt->ip,
-		mydummy, MYDUMMY_SIZE);
+		ipv6string, sizeof(ipv6string));
 #else
 		inetntoa((char *)&acptr->acpt->ip);
 #endif
@@ -1831,7 +1852,7 @@ static void report_fd(aClient *sptr, aClient *acptr, char *to)
 #ifdef INET6
 		inetntop(AF_INET6,
 		(char *)&acptr->ip,
-		mydummy, MYDUMMY_SIZE),
+		ipv6string, sizeof(ipv6string)),
 #else
 		inetntoa((char *)&acptr->ip),
 #endif
@@ -1889,10 +1910,13 @@ int	m_stats(aClient *cptr, aClient *sptr, int parc, char *parv[])
 	doall = !match(name, ME) && !match(cm, ME);
 	wilds = index(cm, '*') || index(cm, '?') || index(cm, '#');
 
+#if 0
+/* That's useless. Why bother? */
 	if (parc > 1 && parv[1][1] != '\0')
 	{
 		stat = '*';
 	}
+#endif
 
 	switch (stat)
 	{
@@ -1900,7 +1924,7 @@ int	m_stats(aClient *cptr, aClient *sptr, int parc, char *parv[])
 		/* send list of file descriptors 
 		 * Avaible only for local opers for security reasons.
 		 */
-		if (!IsAnOper(sptr) || !MyConnect(sptr))
+		if (!is_allowed(sptr, ACL_TRACE) || !MyConnect(sptr))
 		{
 			stat = '*';
 			break;
@@ -1945,6 +1969,9 @@ int	m_stats(aClient *cptr, aClient *sptr, int parc, char *parv[])
 			    {
 				if (!(acptr = local[i]))
 					continue;
+				if (IsUnknown(acptr) && 
+					!(IsAnOper(sptr) && SendWallops(sptr)))
+					continue;
 				if (IsPerson(acptr) && (!MyConnect(sptr)
 				    || !is_allowed(sptr, ACL_TRACE)) && acptr != sptr)
 					continue;
@@ -1986,7 +2013,7 @@ int	m_stats(aClient *cptr, aClient *sptr, int parc, char *parv[])
 					CONF_NOCONNECT_SERVER);
 		break;
 	case 'd' : case 'D' : /* defines */
-		send_defines(cptr, parv[0]);
+		send_defines(cptr, parv[0], parv[1]);
 		break;
 	case 'H' : case 'h' : /* H, L and D conf lines */
 		report_configured_links(cptr, parv[0],
@@ -2374,6 +2401,13 @@ int	m_connect(aClient *cptr, aClient *sptr, int parc, char *parv[])
 		return m_nopriv(cptr, sptr, parc, parv);
 	    }
 
+	if ((bootopt & BOOT_STANDALONE))
+	{
+		sendto_one(sptr, ":%s NOTICE %s :Connect disabled, running in"
+			" standalone mode", ME, parv[0]);
+		return 0;
+	}
+
 	if (hunt_server(cptr,sptr,":%s CONNECT %s %s :%s",
 		       3,parc,parv) != HUNTED_ISME)
 		return 1;
@@ -2788,13 +2822,9 @@ int	m_trace(aClient *cptr, aClient *sptr, int parc, char *parv[])
 				continue;   /* then don't show the client */
 			}
 			
-			/* Report unknown connections to local users
-			 * and remote opers with +w set */
-			if (IsUnknown(a2cptr)
-			    && !((IsAnOper(sptr) || MyClient(sptr))
-				 && SendWallops(sptr)
-				)
-			    )
+			/* Report unknown connections to opers with +w set */
+			if (IsUnknown(a2cptr) &&
+				!(IsAnOper(sptr) && SendWallops(sptr)))
 			{
 				continue;
 			}
@@ -2807,6 +2837,102 @@ int	m_trace(aClient *cptr, aClient *sptr, int parc, char *parv[])
 	return 2;
 }
 
+/*
+ * m_etrace
+ * 	parv[0] = sender prefix
+ */
+int	m_etrace(aClient *cptr, aClient *sptr, int parc, char *parv[])
+{
+	aClient *acptr;
+	int i = 0;
+
+	if (!MyClient(sptr) || !is_allowed(sptr, ACL_TRACE))
+		return m_nopriv(cptr, sptr, parc, parv);
+
+	if (parc > 1)
+	{
+		if ((acptr = find_person(parv[1], NULL)) && MyClient(acptr))
+			sendto_one(sptr, replies[RPL_ETRACEFULL],
+				ME, sptr->name,
+				IsAnOper(acptr) ? "Oper" : "User",
+				get_client_class(acptr),
+				acptr->name, acptr->user->username,
+				acptr->user->host, acptr->user->sip,
+#ifdef XLINE
+				acptr->user2, acptr->user3, 
+#else
+				"-", "-",
+#endif
+				acptr->info);
+	}
+	else
+	{
+		for (i = 0; i <= highest_fd; i++)
+		{
+			if (!(acptr = local[i]))
+				continue;
+
+			if (!IsPerson(acptr))
+				continue;
+		
+			sendto_one(sptr, replies[RPL_ETRACEFULL],
+				ME, sptr->name, 
+				IsAnOper(acptr) ? "Oper" : "User", 
+				get_client_class(acptr), 
+				acptr->name, acptr->user->username, 
+				acptr->user->host, acptr->user->sip,
+#ifdef XLINE
+				acptr->user2, acptr->user3, 
+#else
+				"-", "-",
+#endif
+				acptr->info);
+		}
+	}
+
+	sendto_one(sptr, replies[RPL_ETRACEEND], ME, sptr->name, ME,
+			version, debugmode);
+	return 2;
+}
+
+#ifdef ENABLE_SIDTRACE
+int	m_sidtrace(aClient *cptr, aClient *sptr, int parc, char *parv[])
+{
+	aClient *acptr;
+
+	if (!MyClient(sptr) || !is_allowed(sptr, ACL_SIDTRACE))
+		return m_nopriv(cptr, sptr, parc, parv);
+
+	for (acptr = client; acptr; acptr = acptr->next)
+	{
+		if (!IsPerson(acptr))
+			continue;
+
+		if (strncmp(acptr->user->uid, me.serv->sid, SIDLEN-1))
+			continue;
+
+		sendto_one(sptr, replies[RPL_ETRACEFULL],
+			ME, sptr->name,
+			IsAnOper(acptr) ? "Oper" : "User", 
+			MyClient(acptr) ? get_client_class(acptr) : -1, 
+			acptr->name, acptr->user->username,
+			acptr->user->host, acptr->user->sip, 
+#ifdef XLINE
+			MyClient(acptr) ? acptr->user2 : "-",
+			MyClient(acptr) ? acptr->user3 : "-",
+#else
+			"-", "-",
+#endif
+			acptr->info);
+	}
+
+	sendto_one(sptr, replies[RPL_ETRACEEND], ME, sptr->name, "*",
+			version, debugmode);
+
+	return 3;
+}
+#endif
+
 /*
 ** m_motd
 **	parv[0] = sender prefix
@@ -3034,6 +3160,11 @@ int	m_eob(aClient *cptr, aClient *sptr, int parc, char *parv[])
 int	m_eoback(aClient *cptr, aClient *sptr, int parc, char *parv[])
 {
 	cptr->flags &= ~FLAGS_CBURST;
+	if (cptr->serv && cptr->serv->byuid[0])
+	{
+		/* no need to report link break few days later, is there? --B. */
+		cptr->serv->byuid[0] = '\0';
+	}
 	return 0;
 }
 
@@ -3174,6 +3305,11 @@ int	m_set(aClient *cptr, aClient *sptr, int parc, char *parv[])
 				else if (!mycmp(parv[2], "ON"))
 				{
 					iconf.caccept = 1;
+					/* Well... give admin the chance. */
+					if (!firstrejoindone)
+					{
+						activate_delayed_listeners();
+					}
 				}
 				else if (!mycmp(parv[2], "SPLIT"))
 				{
