@@ -22,7 +22,7 @@
  */
 
 #ifndef lint
-static const volatile char rcsid[] = "@(#)$Id: s_service.c,v 1.65 2005/02/22 17:09:37 chopin Exp $";
+static const volatile char rcsid[] = "@(#)$Id: s_service.c,v 1.69 2010/08/12 01:08:02 bif Exp $";
 #endif
 
 #include "os.h"
@@ -55,7 +55,7 @@ aService	*make_service(aClient *cptr)
 
 void	free_service(aClient *cptr)
 {
-	Reg	aService	*serv;
+	aService	*serv;
 
 	if ((serv = cptr->service))
 	{
@@ -108,7 +108,7 @@ static	aClient *best_service(char *name, aClient *cptr)
 **	action	type on notice
 **	server	origin
 */
-void	check_services_butone(long action, char *server, aClient *cptr,
+void	check_services_butone(long action, aServer *servp, aClient *cptr,
 		char *fmt, ...)
 /* shouldn't cptr be named sptr? */
 {
@@ -128,7 +128,8 @@ void	check_services_butone(long action, char *server, aClient *cptr,
 		** wanted AND if it comes from a server matching the dist
 		*/
 		if ((sp->wants & action)
-		    && (!server || !match(sp->dist, server)))
+		    && (!servp || !match(sp->dist, servp->bcptr->name)
+			|| !match(sp->dist, servp->sid)))
 		{
 			if ((sp->wants & (SERVICE_WANT_PREFIX|SERVICE_WANT_UID))
 			    && cptr && IsRegisteredUser(cptr) &&
@@ -246,7 +247,8 @@ void	check_services_num(aClient *sptr, char *umode)
 		** wanted AND if it comes from a server matching the dist
 		*/
 		if ((sp->wants & SERVICE_MASK_NUM)
-		    && !match(sp->dist, sptr->user->server))
+		    && (!match(sp->dist, sptr->user->server)
+				|| !match(sp->dist, sptr->user->servp->sid)))
 		{
 			sendnum_toone(sp->bcptr, sp->wants, sptr,
 				      umode);
@@ -369,7 +371,7 @@ int	m_service(aClient *cptr, aClient *sptr, int parc, char *parv[])
 				    get_client_name(cptr, FALSE));
 			return exit_client(NULL, acptr, &me, "No Such Server");
 		}
-		if (match(dist, ME))
+		if (match(dist, ME) && match(dist, me.serv->sid))
 		{
 			sendto_flag(SCH_ERROR,
                        	    "ERROR: SERVICE:%s DIST:%s from %s", acptr->name,
@@ -496,7 +498,7 @@ int	m_service(aClient *cptr, aClient *sptr, int parc, char *parv[])
 		if (!(bcptr = local[fdas.fd[i]]) || !IsServer(bcptr) ||
 		    bcptr == cptr)
 			continue;
-		if (match(dist, bcptr->name))
+		if (match(dist, bcptr->name) && match(dist, bcptr->serv->sid))
 			continue;
 
 		sendto_one(bcptr, ":%s SERVICE %s %s %d :%s",
@@ -592,7 +594,8 @@ int	m_servset(aClient *cptr, aClient *sptr, int parc, char *parv[])
 		    {
 			if (!IsServer(acptr) && !IsMe(acptr))
 				continue;
-			if (match(sptr->service->dist, acptr->name))
+			if (match(sptr->service->dist, acptr->name) &&
+					match(sptr->service->dist, acptr->serv->sid))
 				continue;
 			split = (MyConnect(acptr) &&
 				 mycmp(acptr->name, acptr->sockhost));
@@ -616,7 +619,9 @@ int	m_servset(aClient *cptr, aClient *sptr, int parc, char *parv[])
 			if (IsPerson(acptr))
 			    {
 				if (match(sptr->service->dist,
-					  acptr->user->server))
+					  acptr->user->server) &&
+					match(sptr->service->dist,
+					acptr->user->servp->sid))
 					continue;
 				if (burst & SERVICE_WANT_UMODE)
 					send_umode(NULL, acptr, 0, SEND_UMODES,
@@ -631,7 +636,9 @@ int	m_servset(aClient *cptr, aClient *sptr, int parc, char *parv[])
 				if (!(burst & SERVICE_WANT_SERVICE))
 					continue;
 				if (match(sptr->service->dist,
-					  acptr->service->server))
+					  acptr->service->server) &&
+					match(sptr->service->dist,
+					acptr->service->servp->sid))
 					continue;
 				sendto_one(sptr, "SERVICE %s %s %s %d %d :%s",
 					   acptr->name, acptr->service->server,
@@ -699,6 +706,10 @@ int	m_squery(aClient *cptr, aClient *sptr, int parc, char *parv[])
 			sendto_one(acptr, ":%s!%s@%s SQUERY %s :%s", parv[0],
 				   sptr->user->username, sptr->user->host,
 				   acptr->name, parv[2]);
+		else if (MyConnect(acptr) && 
+			(acptr->service->wants & SERVICE_WANT_UID))
+			sendto_one(acptr, ":%s SQUERY %s :%s", sptr->user->uid,
+				   acptr->name, parv[2]);
 		else
 			sendto_one(acptr, ":%s SQUERY %s :%s",
 				   parv[0], acptr->name, parv[2]);
