@@ -22,7 +22,7 @@
  */
 
 #ifndef lint
-static const volatile char rcsid[] = "@(#)$Id: s_user.c,v 1.259 2005/04/13 23:10:37 chopin Exp $";
+static const volatile char rcsid[] = "@(#)$Id: s_user.c,v 1.275 2008/06/11 18:27:27 chopin Exp $";
 #endif
 
 #include "os.h"
@@ -207,7 +207,8 @@ int	hunt_server(aClient *cptr, aClient *sptr, char *command, int server,
 			parv[server] = acptr->name;
 		if (IsService(sptr)
 		    && (IsServer(acptr->from)
-			&& match(sptr->service->dist,acptr->name) != 0))
+			&& match(sptr->service->dist,acptr->name) != 0
+			&& match(sptr->service->dist,acptr->serv->sid) != 0))
 		    {
 			sendto_one(sptr, replies[ERR_NOSUCHSERVER], ME, BadTo(parv[0]), 
 				   parv[server]);
@@ -252,6 +253,11 @@ int	do_nick_name(char *nick, int server)
 	if (strcasecmp(nick, "anonymous") == 0)
 		return 0;
 
+#ifdef MINLOCALNICKLEN
+	if (!server && nick[MINLOCALNICKLEN-1] == '\0')
+		return 0;
+#endif
+
 	for (ch = nick; *ch && (ch-nick) < (server?NICKLEN:LOCALNICKLEN); ch++)
 	{
 		if (!isvalidnick(*ch))
@@ -327,6 +333,11 @@ int	register_user(aClient *cptr, aClient *sptr, char *nick, char *username)
 	char	prefix;
 #endif
 	int	i;
+#ifdef XLINE
+	static char savedusername[USERLEN+1];
+
+	strncpyzt(savedusername, username, USERLEN+1);
+#endif
 
 	user->last = timeofday;
 	parv[0] = sptr->name;
@@ -338,6 +349,59 @@ int	register_user(aClient *cptr, aClient *sptr, char *nick, char *username)
 #ifdef RESTRICT_USERNAMES
 		char *lbuf = NULL;
 #endif
+#ifdef XLINE
+		aConfItem *xtmp;
+
+		/* Just for clarification, so there's less confusion:
+		   X-lines read from config have their fields stored
+		   in aConf in the following fields:
+		   host, passwd, name, name2, name3, source_ip.
+		   (see s_conf.c/initconf(), look for XLINE);
+		   these are used to check against USER and NICK
+		   commands parameters during registration:
+		   USER 1st 2nd 3rd :4th
+		   NICK 5th
+		   additionally user ip and/or hostname are
+		   being matched against X-line last field
+		   (conveniently kept in aconf->source_ip). --B. */
+
+		for (xtmp = conf; xtmp; xtmp = xtmp->next)
+		{
+			if (xtmp->status != CONF_XLINE)
+				continue;
+			if (!BadPtr(xtmp->host) && 
+				match(xtmp->host, username))
+				continue;
+			if (!BadPtr(xtmp->passwd) && 
+				match(xtmp->passwd, sptr->user2))
+				continue;
+			if (!BadPtr(xtmp->name) && 
+				match(xtmp->name, sptr->user3))
+				continue;
+			if (!BadPtr(xtmp->name2) && 
+				match(xtmp->name2, sptr->info))
+				continue;
+			if (!BadPtr(xtmp->name3) && 
+				match(xtmp->name3, nick))
+				continue;
+			if (!BadPtr(xtmp->source_ip) &&
+				(match(xtmp->source_ip, (sptr->hostp ?
+				sptr->hostp->h_name : sptr->sockhost)) &&
+				match(xtmp->source_ip, sptr->user->sip) &&
+				strchr(xtmp->source_ip, '/') && 
+				match_ipmask(xtmp->source_ip, sptr, 0)))
+				continue;
+			SetXlined(sptr);
+			break;
+		}
+
+		if (IsXlined(sptr))
+		{
+			sptr->exitc = EXITC_XLINE;
+			return exit_client(cptr, sptr, &me,
+				XLINE_EXIT_REASON);
+		}
+#endif
 
 #if defined(USE_IAUTH)
 		static time_t last = 0;
@@ -575,14 +639,6 @@ int	register_user(aClient *cptr, aClient *sptr, char *nick, char *username)
 			return exit_client(cptr, sptr, &me,
 				(reason) ? buf : "K-lined");
 		}
-#ifdef XLINE
-		if (IsXlined(sptr))
-		{
-			sptr->exitc = EXITC_XLINE;
-			return exit_client(cptr, sptr, &me,
-				XLINE_EXIT_REASON);
-		}
-#endif
 	}
 	else
 	{
@@ -673,11 +729,17 @@ int	register_user(aClient *cptr, aClient *sptr, char *nick, char *username)
 # if defined(CLIENTS_CHANNEL) && (CLIENTS_CHANNEL_LEVEL & CCL_CONN)
 		sendto_flag(SCH_CLIENT, "%s %s %s %s CONN %s"
 # if (CLIENTS_CHANNEL_LEVEL & CCL_CONNINFO)
+#  ifdef XLINE
+         " %s %s %s"
+#  endif
 			" :%s"
 # endif
 			, user->uid, nick, user->username,
 			user->host, user->sip
 # if (CLIENTS_CHANNEL_LEVEL & CCL_CONNINFO)
+#  ifdef XLINE
+         , savedusername, sptr->user2, sptr->user3
+#  endif
 			, sptr->info
 # endif
 			);
@@ -748,21 +810,26 @@ int	register_user(aClient *cptr, aClient *sptr, char *nick, char *username)
 	}	/* for(my-leaf-servers) */
 #ifdef	USE_SERVICES
 #if 0
-	check_services_butone(SERVICE_WANT_NICK, user->server, NULL,
+	check_services_butone(SERVICE_WANT_NICK, user->servp, NULL,
 			      "NICK %s :%d", nick, sptr->hopcount+1);
-	check_services_butone(SERVICE_WANT_USER, user->server, sptr,
+	check_services_butone(SERVICE_WANT_USER, user->servp, sptr,
 			      ":%s USER %s %s %s :%s", nick, user->username, 
 			      user->host, user->server, sptr->info);
 	if (MyConnect(sptr))	/* all modes about local users */
 		send_umode(NULL, sptr, 0, ALL_UMODES, buf);
-	check_services_butone(SERVICE_WANT_UMODE, user->server, sptr,
+	check_services_butone(SERVICE_WANT_UMODE, user->servp, sptr,
 			      ":%s MODE %s :%s", nick, nick, buf);
 #endif
 	if (MyConnect(sptr))	/* all modes about local users */
 		send_umode(NULL, sptr, 0, ALL_UMODES, buf);
 	check_services_num(sptr, buf);
 #endif
+#ifdef USE_HOSTHASH
 	add_to_hostname_hash_table(user->host, user);
+#endif
+#ifdef USE_IPHASH
+	add_to_ip_hash_table(user->sip, user);
+#endif
 	return 1;
 }
 
@@ -1104,7 +1171,7 @@ nickkilldone:
 			add_history(sptr, sptr);
 #ifdef	USE_SERVICES
 			check_services_butone(SERVICE_WANT_NICK,
-					      sptr->user->server, sptr,
+					      sptr->user->servp, sptr,
 					      ":%s NICK :%s", parv[0], nick);
 #endif
 		}
@@ -1413,7 +1480,8 @@ static	int	m_message(aClient *cptr, aClient *sptr, int parc,
 		/*
 		** nickname addressed?
 		*/
-		if ((IsServer(cptr) && (acptr = find_uid(nick, NULL))) || 
+		if (((IsServer(cptr) || IsService(cptr))
+			&& (acptr = find_uid(nick, NULL))) || 
 			(acptr = find_person(nick, NULL)))
 		    {
 			if (!notice && MyConnect(sptr) &&
@@ -1635,7 +1703,7 @@ static	void	who_one(aClient *sptr, aClient *acptr, aChannel *repchan,
 	sendto_one(sptr, replies[RPL_WHOREPLY], ME, BadTo(sptr->name),
 		   (repchan) ? (repchan->chname) : "*", acptr->user->username,
 		   acptr->user->host, acptr->user->server, acptr->name,
-		   status, acptr->hopcount, acptr->info);
+		   status, acptr->hopcount, acptr->user->servp->sid, acptr->info);
 }
 
 
@@ -1687,6 +1755,7 @@ static	void	who_find(aClient *sptr, char *mask, int oper)
 	aClient	*acptr;
 	
 	/* first, show INvisible matching users on common channels */
+	if (sptr->user) /* service can request who as well */
 	for (lp = sptr->user->channel; lp ;lp = lp->next)
 	{
 		chptr = lp->value.chptr;
@@ -2176,28 +2245,6 @@ int	m_user(aClient *cptr, aClient *sptr, int parc, char *parv[])
 #ifdef	DEFAULT_INVISIBLE
 	SetInvisible(sptr);
 #endif
-#ifdef XLINE
-	{
-		aConfItem *tmp;
-
-		for (tmp = conf; tmp; tmp = tmp->next)
-		{
-			if (tmp->status != CONF_XLINE)
-				continue;
-			if (!BadPtr(tmp->source_ip) && match(tmp->source_ip, realname))
-				continue;
-			if (!BadPtr(tmp->host) && match(tmp->host, username))
-				continue;
-			if (!BadPtr(tmp->passwd) && match(tmp->passwd, umodes))
-				continue;
-			if (!BadPtr(tmp->name) && match(tmp->name, server))
-				continue;
-			SetXlined(sptr);
-			break;
-               }
-       }
-#endif
-
 	/* parse desired user modes sent in USER */
 	/* rfc behaviour - bits */
 	if (isdigit(*umodes))
@@ -2259,6 +2306,10 @@ int	m_user(aClient *cptr, aClient *sptr, int parc, char *parv[])
 	if (strlen(realname) > REALLEN)
 		realname[REALLEN] = '\0';
 	sptr->info = mystrdup(realname);
+#ifdef XLINE
+	sptr->user2 = mystrdup(umodes);
+	sptr->user3 = mystrdup(server);
+#endif
 	if (sptr->name[0]) /* NICK already received, now we have USER... */
 	{
 		return register_user(cptr, sptr, sptr->name, username);
@@ -2731,7 +2782,7 @@ int	m_oper(aClient *cptr, aClient *sptr, int parc, char *parv[])
 		send_umode_out(cptr, sptr, old);
  		sendto_one(sptr, replies[RPL_YOUREOPER], ME, BadTo(parv[0]));
 #ifdef	USE_SERVICES
-		check_services_butone(SERVICE_WANT_OPER, sptr->user->server, 
+		check_services_butone(SERVICE_WANT_OPER, sptr->user->servp, 
 				      sptr, ":%s MODE %s :+%c", parv[0],
 				      parv[0], IsOper(sptr) ? 'o' : 'O');
 #endif
@@ -2742,10 +2793,14 @@ int	m_oper(aClient *cptr, aClient *sptr, int parc, char *parv[])
 		}
 		logstring = "";
 	}	
-	else /* Wrong password */
+	else /* Wrong password or attach_conf() failed */
 	{
 		(void)detach_conf(sptr, aconf);
-		sendto_one(sptr,replies[ERR_PASSWDMISMATCH], ME, BadTo(parv[0]));
+		if (!StrEq(encr, aconf->passwd))
+			sendto_one(sptr,replies[ERR_PASSWDMISMATCH], ME, BadTo(parv[0]));
+		else
+			sendto_one(sptr,":%s %d %s :Too many connections",
+				ME, ERR_PASSWDMISMATCH, BadTo(parv[0]));
 #ifdef FAILED_OPERLOG
 		sendto_flag(SCH_NOTICE, "FAILED OPER attempt by %s!%s@%s",
 			parv[0], sptr->user->username, sptr->user->host);
@@ -2764,8 +2819,8 @@ int	m_oper(aClient *cptr, aClient *sptr, int parc, char *parv[])
 			IsUnixSocket(sptr) ? sptr->sockhost :
 #endif
 #ifdef INET6
-                       inet_ntop(AF_INET6, (char *)&sptr->ip, mydummy,
-			       MYDUMMY_SIZE)
+                       inet_ntop(AF_INET6, (char *)&sptr->ip, ipv6string,
+			       sizeof(ipv6string))
 #else
                        inetntoa((char *)&sptr->ip)
 #endif
@@ -2805,7 +2860,7 @@ int	m_oper(aClient *cptr, aClient *sptr, int parc, char *parv[])
 #endif
 #ifdef INET6
 				inetntop(AF_INET6, (char *)&sptr->ip,
-					mydummy, MYDUMMY_SIZE)
+					ipv6string, sizeof(ipv6string))
 #else
 				inetntoa((char *)&sptr->ip)
 #endif
@@ -3049,14 +3104,14 @@ int	m_umode(aClient *cptr, aClient *sptr, int parc, char *parv[])
 					sptr->user->away = NULL;
 #ifdef  USE_SERVICES
 				check_services_butone(SERVICE_WANT_AWAY,
-						      sptr->user->server, sptr,
+						      sptr->user->servp, sptr,
 						      ":%s AWAY", parv[0]);
 #endif
 				    }
 #ifdef  USE_SERVICES
 				if (what == MODE_ADD)
 				check_services_butone(SERVICE_WANT_AWAY,
-						      sptr->user->server, sptr,
+						      sptr->user->servp, sptr,
 						      ":%s AWAY :", parv[0]);
 #endif
 			default :
@@ -3123,7 +3178,7 @@ int	m_umode(aClient *cptr, aClient *sptr, int parc, char *parv[])
 		istat.is_oper++;
 		sptr->user->servp->usercnt[2]++;
 #ifdef	USE_SERVICES
-		check_services_butone(SERVICE_WANT_OPER, sptr->user->server,
+		check_services_butone(SERVICE_WANT_OPER, sptr->user->servp,
 				      sptr, ":%s MODE %s :+o", parv[0],
 				      parv[0]);
 #endif
@@ -3133,7 +3188,7 @@ int	m_umode(aClient *cptr, aClient *sptr, int parc, char *parv[])
 		istat.is_oper--;
 		sptr->user->servp->usercnt[2]--;
 #ifdef	USE_SERVICES
-		check_services_butone(SERVICE_WANT_OPER, sptr->user->server,
+		check_services_butone(SERVICE_WANT_OPER, sptr->user->servp,
 				      sptr, ":%s MODE %s :-o", parv[0],
 				      parv[0]);
 #endif
@@ -3143,7 +3198,7 @@ int	m_umode(aClient *cptr, aClient *sptr, int parc, char *parv[])
 		istat.is_oper--;
 		sptr->user->servp->usercnt[2]--;
 #ifdef USE_SERVICES
-		check_services_butone(SERVICE_WANT_OPER, sptr->user->server,
+		check_services_butone(SERVICE_WANT_OPER, sptr->user->servp,
 				      sptr, ":%s MODE %s :-O", parv[0],
 				      parv[0]);
 #endif
@@ -3230,7 +3285,7 @@ void	send_umode_out(aClient *cptr, aClient *sptr, int old)
 #ifdef USE_SERVICES
 	/* buf contains all modes for local users, and iow only for remotes */
 	if (*buf)
-		check_services_butone(SERVICE_WANT_UMODE, sptr->user->server,
+		check_services_butone(SERVICE_WANT_UMODE, sptr->user->servp,
 				      sptr, ":%s MODE %s :%s", sptr->name,
 				      sptr->name, buf);
 #endif
@@ -3263,7 +3318,7 @@ static	void	save_user(aClient *cptr, aClient *sptr, char *path)
 			       sptr->name, sptr->user->uid);
 	add_history(sptr, NULL);
 #ifdef	USE_SERVICES
-	check_services_butone(SERVICE_WANT_NICK, sptr->user->server, sptr,
+	check_services_butone(SERVICE_WANT_NICK, sptr->user->servp, sptr,
 			      ":%s NICK :%s", sptr->name, sptr->user->uid);
 #endif
 	sendto_serv_v(cptr, SV_UID, ":%s SAVE %s :%s%c%s", 
@@ -3324,6 +3379,9 @@ int	is_allowed(aClient *cptr, long function)
 		if (function == ACL_TKLINE &&
 			(cptr->service->wants & SERVICE_WANT_TKLINE))
 			return 1;
+		if (function == ACL_KLINE &&
+			(cptr->service->wants & SERVICE_WANT_KLINE))
+			return 1;
 		return 0;
 	}
 
