@@ -27,12 +27,15 @@
 #define	SERVICE_WANT_USERLOG	0x10000000 /* FNAME_USERLOG */
 #define	SERVICE_WANT_CONNLOG	0x20000000 /* FNAME_CONNLOG */
 
-#define	SERVICE_WANT_TKLINE	0x40000000 /* service wants to TKLINE */
+#define	SERVICE_WANT_CLIENTS	0x40000000 /* &CLIENTS */
+
+#define	SERVICE_WANT_TKLINE	0x00100000 /* service wants to TKLINE */
+#define	SERVICE_WANT_KLINE	0x00200000 /* service wants to KLINE */
 
 /* masks */
 #define	SERVICE_MASK_GLOBAL	0x00007000 /*for these,service must be global*/
 #define	SERVICE_MASK_PREFIX	0x00000FFF /* these actions have a prefix */
-#define	SERVICE_MASK_ALL	0x7F00FFFF /* all possible actions */
+#define	SERVICE_MASK_ALL	0x7F30FFFF /* all possible actions */
 #define	SERVICE_MASK_NUM	(SERVICE_WANT_NICK|SERVICE_WANT_USER|\
 				 SERVICE_WANT_UMODE)
 
